from .logger import LoggerManager

c_logger = LoggerManager().coder
r_logger = LoggerManager().researcher
p_logger = LoggerManager().planner
e_logger = LoggerManager().executor
s_logger = LoggerManager().sirji
u_logger = LoggerManager().user
